package cn.hh.common.restful;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.NoHandlerFoundException;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Path;
import javax.validation.ValidationException;
import java.util.List;
import java.util.Objects;
import java.util.Set;


/**
 * @author shisl
 * @package cn.hh.common.restful
 * @class GlobalExceptionHandler
 * @date 2022/12/20 上午11:16
 * @description 全局异常处理器
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    /**
     * 处理自定义异常
     *
     * @param e
     * @return
     */
    @ExceptionHandler(BusinessException.class)
    public RestResult<Object> handleBusinessException(BusinessException e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(e.getCode(), e.getMessage());
    }

    /**
     * 统一处理请求参数校验(实体对象传参)
     *
     * @param e BindException
     * @return
     */
    @ExceptionHandler(BindException.class)
    public RestResult<Object> validExceptionHandler(BindException e) {
        StringBuilder message = new StringBuilder();
        List<FieldError> fieldErrors = e.getBindingResult().getFieldErrors();
        for (FieldError error : fieldErrors) {
            message.append(error.getField()).append(error.getDefaultMessage()).append(",");
        }
        message = new StringBuilder(message.substring(0, message.length() - 1));
        return RestResponse.build(RestCode.FAIL.getCode(), message.toString());
    }

    /**
     * 处理方法校验
     *
     * @param e
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public RestResult<Object> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(RestCode.FAIL.getCode(), Objects.requireNonNull(e.getBindingResult().getFieldError()).getDefaultMessage());
    }

    /**
     * 处理参数校验
     *
     * @param e
     * @return
     */
    @ExceptionHandler(ValidationException.class)
    public RestResult<Object> handleValidationException(ValidationException e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(RestCode.METHOD_ARGUMENT_NOT_VALID_EXCEPTION.getCode(), e.getCause().getMessage());
    }

    /**
     * 统一处理请求参数校验(普通传参)
     *
     * @param e ConstraintViolationException
     * @return
     */
    @ExceptionHandler(value = ConstraintViolationException.class)
    public RestResult<Object> handleConstraintViolationException(ConstraintViolationException e) {
        StringBuilder message = new StringBuilder();
        Set<ConstraintViolation<?>> violations = e.getConstraintViolations();
        for (ConstraintViolation<?> violation : violations) {
            Path path = violation.getPropertyPath();
            String[] pathArr = StringUtils.splitByWholeSeparatorPreserveAllTokens(path.toString(), ".");
            message.append(String.format("[%s]", pathArr[1])).append(violation.getMessage()).append(";");
        }
        message = new StringBuilder(message.substring(0, message.length() - 1));
        return RestResponse.build(RestCode.FAIL.getCode(), message.toString());
    }

    @ExceptionHandler(NoHandlerFoundException.class)
    public RestResult<Object> handlerNoFoundException(Exception e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(RestCode.NOT_FOUND.getCode(), RestCode.NOT_FOUND.getMsg());
    }

    /**
     * 处理数据重复
     *
     * @param e
     * @return
     */
    @ExceptionHandler(DuplicateKeyException.class)
    public RestResult<Object> handleDuplicateKeyException(DuplicateKeyException e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(RestCode.DUPLICATE_KEY_EXCEPTION.getCode(), RestCode.DUPLICATE_KEY_EXCEPTION.getMsg());
    }


    /**
     * 处理异常
     *
     * @param e
     * @return
     */
    @ExceptionHandler(Exception.class)
    public RestResult<Object> handleException(Exception e) {
        log.error(e.getMessage(), e);
        return RestResponse.build(RestCode.INTERNAL_SERVER_ERROR.getCode(), RestCode.INTERNAL_SERVER_ERROR.getMsg());
    }


}
