package cn.hh.common.utils;

import cn.hh.common.constants.CommonConstants;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.DecodeHintType;
import com.google.zxing.EncodeHintType;
import com.google.zxing.LuminanceSource;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.Result;
import com.google.zxing.client.j2se.BufferedImageLuminanceSource;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author shisl
 * @package cn.hh.common.util
 * @class QrCodeUtils
 * @date 2023/1/14 下午2:47
 * @description 二维码工具类
 */
@Slf4j
public class QrCodeUtils {

    // 二维码颜色
    private static final int BLACK = 0xFF000000;

    private static final int WHITE = 0xFFFFFFFF;


    /**
     * 根据内容，生成指定宽高、指定格式的二维码图片
     *
     * @param text   内容
     * @param width  宽
     * @param height 高
     * @return 生成的二维码图片路径
     * @throws Exception
     */
    public static BufferedImage generateQrCode(String text, int width, int height)
            throws Exception {
        BufferedImage bufferedImage = writeToFile(generateQrCodeBitMatrix(text, width, height));
        return bufferedImage;
    }


    public static BitMatrix generateQrCodeBitMatrix(String text, int width, int height)
            throws Exception {
        Hashtable<EncodeHintType, Object> hints = new Hashtable<>();
        // 指定编码格式
        hints.put(EncodeHintType.CHARACTER_SET, CommonConstants.UTF8);
        // 指定纠错等级
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.L);
        // 白边大小，取值范围0~4
        hints.put(EncodeHintType.MARGIN, 0);
        BitMatrix bitMatrix = new MultiFormatWriter().encode(text, BarcodeFormat.QR_CODE, width, height, hints);
        return bitMatrix;
    }


    /**
     * 生成二维码图片
     *
     * @param matrix
     * @throws IOException
     */
    public static BufferedImage writeToFile(BitMatrix matrix) {
        BufferedImage image = toBufferedImage(matrix);
        return image;
    }

    /**
     * @param matrix
     * @return
     */
    private static BufferedImage toBufferedImage(BitMatrix matrix) {
        int width = matrix.getWidth();
        int height = matrix.getHeight();
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, matrix.get(x, y) ? BLACK : WHITE);
            }
        }
        return image;
    }


    /**
     * 解析二维码图像
     *
     * @param base64Str
     * @return 二维码内容
     */
    public static QRResult decode(String base64Str) {
        return decode(ImageUtil.base2InputStream(base64Str));
    }

    /**
     * 解析二维码图像
     * <p>
     * return 二维码内容
     */
    /**
     * 流图片解码
     */
    public static QRResult decode(InputStream input) {
        BufferedImage image;
        try {
            if (null == input) {
                return new QRResult("得到的文件不存在！", 300);
            }
            image = ImageIO.read(input);
            LuminanceSource source = new BufferedImageLuminanceSource(image);
            BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
            Map<DecodeHintType, Object> hints = new LinkedHashMap<DecodeHintType, Object>();
            // 解码设置编码方式为：utf-8，
            hints.put(DecodeHintType.CHARACTER_SET, "utf-8");
            //优化精度
            hints.put(DecodeHintType.TRY_HARDER, Boolean.TRUE);
            //复杂模式，开启PURE_BARCODE模式
            hints.put(DecodeHintType.PURE_BARCODE, Boolean.TRUE);
            Result result = new MultiFormatReader().decode(bitmap, hints);
            String txt = result.getText();
            return new QRResult("成功解码！", 200, txt);
        } catch (Exception e) {
            log.error("解码失败。", e);
            return new QRResult("解码失败，请确认的你二维码是否正确，或者图片有多个二维码！", 500);
        }
    }

    /**
     * 返回值处理
     */
    @Data
    public static class QRResult {
        public QRResult(String message, int status) {
            this.message = message;
            this.status = status;
            this.txt = "";
        }

        public QRResult(String message, int status, String txt) {
            this.message = message;
            this.status = status;
            this.txt = txt;
        }

        //解码内容
        private String txt; //返回的消息内容
        private String message; //返回的状态码，200：成功，500：错误
        private int status;
    }


}
