package cn.hh.hhface.config;

import cn.hh.common.constants.CommonConstants;
import cn.hh.common.restful.BusinessException;
import cn.hh.common.utils.Preconditions;
import cn.hh.common.utils.RequestUriUtils;
import cn.hh.hhface.service.WebsocketService;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.websocketx.CloseWebSocketFrame;
import io.netty.handler.codec.http.websocketx.PingWebSocketFrame;
import io.netty.handler.codec.http.websocketx.PongWebSocketFrame;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import io.netty.handler.codec.http.websocketx.WebSocketFrame;
import io.netty.handler.codec.http.websocketx.WebSocketServerHandshaker;
import io.netty.handler.codec.http.websocketx.WebSocketServerHandshakerFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package cn.hh.hhface.config
 * @class NioWebSocketHandler
 * @date 2023/12/10 下午6:03
 * @description WebSocket连接数据接收处理类
 */
@Slf4j
@Sharable
@Component
public class NioWebSocketHandler extends SimpleChannelInboundHandler<WebSocketFrame> {

    @Autowired
    private NioWebSocketChannelPool webSocketChannelPool;
    @Autowired
    private WebSocketProperties webSocketProperties;
    @Resource
    private RedisTemplate<String, String> redisTemplate;
    @Resource
    private WebsocketService websocketService;

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("客户端连接：{}", ctx.channel().id());
        webSocketChannelPool.addChannel(ctx.channel());
        super.channelActive(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.info("客户端断开连接：{}", ctx.channel().id());
        webSocketChannelPool.removeChannel(ctx.channel());
        super.channelInactive(ctx);
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) {
        ctx.channel().flush();
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, WebSocketFrame frame) {
        // 根据请求数据类型进行分发处理
        if (frame instanceof PingWebSocketFrame) {
            pingWebSocketFrameHandler(ctx, (PingWebSocketFrame) frame);
        } else if (frame instanceof TextWebSocketFrame) {
            textWebSocketFrameHandler(ctx, (TextWebSocketFrame) frame);
        } else if (frame instanceof CloseWebSocketFrame) {
            closeWebSocketFrameHandler(ctx, (CloseWebSocketFrame) frame);
        }
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
//        log.info("客户端请求数据类型：{}", msg.getClass());
        if (msg instanceof FullHttpRequest) {
            fullHttpRequestHandler(ctx, (FullHttpRequest) msg);
        }
        super.channelRead(ctx, msg);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.error("socket连接异常:{}", cause.getMessage());
        Map<String, Object> data = new HashMap<>();
        data.put("code", 400);
        data.put("msg", cause.getMessage());
        ctx.channel().writeAndFlush(new TextWebSocketFrame(JSON.toJSONString(data)));
        ctx.channel().close();
    }

    /**
     * 处理连接请求，客户端WebSocket发送握手包时会执行这一次请求
     *
     * @param ctx
     * @param request
     */
    private void fullHttpRequestHandler(ChannelHandlerContext ctx, FullHttpRequest request) {
        String uri = request.uri();
        Map<String, String> params = RequestUriUtils.getParams(uri);
        log.info("客户端请求参数：{}", params);
        String errorMsg = null;
        String token = params.get(CommonConstants.WEBSOCKET_CONNECT_PC);
        if (Preconditions.isBlank(params.get(CommonConstants.WEBSOCKET_CONNECT_APP)) && Preconditions.isBlank(token)) {
            errorMsg = "参数有误，无权访问websocket资源";
        }

        //消息来源
        String source = params.get(CommonConstants.WEBSOCKET_CONNECT_SOURCE);
        if (Preconditions.isNotBlank(source) && !Arrays.asList("TQ", "TQSERVER", "HHFACE").contains(source)) {
            errorMsg = "source参数有误，无权访问websocket资源";
        }

        // 判断请求路径是否跟配置中的一致
        if (Preconditions.isBlank(errorMsg) && webSocketProperties.getPath().equals(RequestUriUtils.getBasePath(uri))) {
            // 因为有可能携带了参数，导致客户端一直无法返回握手包，因此在校验通过后，重置请求路径
            request.setUri(webSocketProperties.getPath());
            //放入缓存
            String sn = params.get(CommonConstants.WEBSOCKET_CONNECT_APP);
            String channelId = ctx.channel().id().asShortText();
            //APP客户端
            if (Preconditions.isNotBlank(sn)) {
                //放入缓存 保存sn和通道的关系
                String cacheChannelKey = Preconditions.isNotBlank(source) ? String.format(CommonConstants.WEBSOCKET_SN_SOURCE_KEY, source, sn) : String.format(CommonConstants.WEBSOCKET_SN_KEY, sn);
                redisTemplate.opsForValue().set(cacheChannelKey, channelId, 365, TimeUnit.DAYS);
            }

            //PC端
            if (Preconditions.isNotBlank(token)) {
                String wpKey = Preconditions.isNotBlank(source) ? CommonConstants.WEBSOCKET_PC_SOURCE_KEY : CommonConstants.WEBSOCKET_PC_KEY;
                String wptKey = Preconditions.isNotBlank(source) ? String.format(CommonConstants.WEBSOCKET_PC_TOKEN_SOURCE_KEY, source, DigestUtil.md5Hex(token)) : String.format(CommonConstants.WEBSOCKET_PC_TOKEN_KEY, DigestUtil.md5Hex(token));
                //放入缓存
                handleTokenCache(channelId, wpKey, wptKey);
            }
        } else {
            log.error(errorMsg);
            //先建立临时连接
            WebSocketServerHandshakerFactory wsFactory = new WebSocketServerHandshakerFactory(uri, null, false);
            WebSocketServerHandshaker handshaker = wsFactory.newHandshaker(request);
            if (handshaker == null) {
                WebSocketServerHandshakerFactory.sendUnsupportedVersionResponse(ctx.channel());
            } else {
                handshaker.handshake(ctx.channel(), request);
                ctx.channel().writeAndFlush(new TextWebSocketFrame("临时连接建立成功"));
            }
            throw new BusinessException(errorMsg);
        }
    }

    //PC放入缓存
    private void handleTokenCache(String channelId, String wpKey, String wptKey) {
        List<String> channels = new ArrayList<>();
        //放入缓存
        if (redisTemplate.hasKey(wpKey)) {
            String s = redisTemplate.opsForValue().get(wpKey);
            List<String> redisChannels = JSONObject.parseArray(s, String.class);
            if (!redisChannels.contains(channelId)) {
                channels.add(channelId);
            }
            //处理webSocketChannelPool中不存在的channel
            List<String> filterChannels = redisChannels.stream().filter(item -> Preconditions.isNotBlank(webSocketChannelPool.findChannel(item))).collect(Collectors.toList());
            if (Preconditions.isNotBlank(filterChannels)) {
                channels.addAll(filterChannels);
            }

            if (Preconditions.isNotBlank(channels)) {
                redisTemplate.opsForValue().set(wpKey, JSONObject.toJSONString(channels), 15, TimeUnit.DAYS);
            }
        } else {
            channels.add(channelId);
            redisTemplate.opsForValue().set(wpKey, JSONObject.toJSONString(channels), 15, TimeUnit.DAYS);
        }
        //保存token和通道对应关系
        redisTemplate.opsForValue().set(wptKey, channelId, 7, TimeUnit.DAYS);
    }

    /**
     * 客户端发送断开请求处理
     *
     * @param ctx
     * @param frame
     */
    private void closeWebSocketFrameHandler(ChannelHandlerContext ctx, CloseWebSocketFrame frame) {
        log.info("客户端发送断开请求:{}", frame.reasonText());
        String channelId = ctx.channel().id().asShortText();
        if (redisTemplate.hasKey(CommonConstants.WEBSOCKET_PC_KEY)) {
            String s = redisTemplate.opsForValue().get(CommonConstants.WEBSOCKET_PC_KEY);
            List<String> channels = JSONObject.parseArray(s, String.class);
            channels = channels.stream().filter(item -> !channelId.equals(item)).collect(Collectors.toList());
            redisTemplate.opsForValue().set(CommonConstants.WEBSOCKET_PC_KEY, JSONObject.toJSONString(channels), 15, TimeUnit.DAYS);
        }
        if (redisTemplate.hasKey(CommonConstants.WEBSOCKET_PC_SOURCE_KEY)) {
            String s = redisTemplate.opsForValue().get(CommonConstants.WEBSOCKET_PC_SOURCE_KEY);
            List<String> channels = JSONObject.parseArray(s, String.class);
            channels = channels.stream().filter(item -> !channelId.equals(item)).collect(Collectors.toList());
            redisTemplate.opsForValue().set(CommonConstants.WEBSOCKET_PC_SOURCE_KEY, JSONObject.toJSONString(channels), 15, TimeUnit.DAYS);
        }
        ctx.close();
    }

    /**
     * 创建连接之后，客户端发送的消息都会在这里处理
     *
     * @param ctx
     * @param frame
     */
    private void textWebSocketFrameHandler(ChannelHandlerContext ctx, TextWebSocketFrame frame) {
        // 客户端发送过来的内容不进行业务处理，原样返回
        log.info("客户端发送的消息内容:{}", frame.text());
        try {
            JSONObject object = JSONObject.parseObject(frame.text());
            //来源  PC / APP
            String source = object.getString(CommonConstants.WEBSOCKET_SOURCE);
            //TODO 处理pc端发送过来的websocket消息
            if (CommonConstants.WEBSOCKET_SOURCE_PC.equals(source)) {
                // TODO 暂时没有直接使用
                ctx.channel().writeAndFlush(frame.retain());
            }
            //TODO 处理APP端发送过来的websocket消息
            if (CommonConstants.WEBSOCKET_SOURCE_APP.equals(source)) {
                handleAppWebsocketMsg(object, frame.text());
            }

            // TODO 来源--塘栖PC端
            if (CommonConstants.WEBSOCKET_SOURCE_TQ_PC.equals(source)) {
                ctx.channel().writeAndFlush(frame.retain());
            }
            //TODO 来源--塘栖服务端
            if (CommonConstants.WEBSOCKET_SOURCE_TQ_SERVER.equals(source)) {
                handleTqServerWebsocketMsg(object, frame.text());
            }
            //TODO 来源--HHFACE端
            if (CommonConstants.WEBSOCKET_SOURCE_HHFACE_APP.equals(source)) {
                handleHhFaceAppWebsocketMsg(object, frame.text());
            }
        } catch (Exception e) {
            ctx.channel().writeAndFlush(frame.retain());
        }
    }

    //TODO 来源--HHFACE端
    private void handleHhFaceAppWebsocketMsg(JSONObject object, String text) {
        String op = object.getString(CommonConstants.WEBSOCKET_OP);
        if (Preconditions.isNotBlank(op)) {
            if (op.equals(CommonConstants.WEBSOCKET_OP_TRACKING)) { //HHFACE心跳包
                handleHHFaceHeartTracking(object, text, op);
                return;
            }
            websocketService.handleHhFaceAppWebsocketOp(object, text, op);
        } else {
            //没有操作类型 全部广播📢
            websocketService.sendHhFaceAppWebsocketMsg2AllTq(text);
        }
    }

    //HHFACE心跳包
    private void handleHHFaceHeartTracking(JSONObject object, String text, String op) {
        object.put("code", 200);
        object.put("msg", "success");
        object.put("source", CommonConstants.WEBSOCKET_SOURCE_TQ_SERVER);
        websocketService.handleTqServerWebsocketOp(object, text, op);
    }

    //TODO 来源--塘栖服务端
    private void handleTqServerWebsocketMsg(JSONObject object, String text) {
        String op = object.getString(CommonConstants.WEBSOCKET_OP);
        if (Preconditions.isNotBlank(op)) {
            websocketService.handleTqServerWebsocketOp(object, text, op);
        }
    }

    private void handleAppWebsocketMsg(JSONObject object, String text) {
        //PC操作 close关闭应用  reboot重启设备  upgrade升级
        //APP操作 heart心跳在离线 downloadProcessing下载进度
        String op = object.getString(CommonConstants.WEBSOCKET_OP);
        if (Preconditions.isNotBlank(op)) {
            websocketService.handleAppWebsocketOp(object, text, op);
        } else {
            //没有操作类型 全部广播📢
            websocketService.sendAppWebsocketMsg2AllPc(text);
        }
    }

    /**
     * 处理客户端心跳包
     *
     * @param ctx
     * @param frame
     */
    private void pingWebSocketFrameHandler(ChannelHandlerContext ctx, PingWebSocketFrame frame) {
        log.info("客户端心跳:{}", frame.content());
        ctx.channel().writeAndFlush(new PongWebSocketFrame(frame.content().retain()));
    }
}
