package com.yx.face.boot.aop;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.annotation.RepeatSubmit;
import com.yx.face.boot.component.security.JWTUtils;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestCode;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.IPUtils;
import com.yx.face.boot.uitls.JsonUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Method;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author shisl
 * @package com.yx.face.boot.aop
 * @class RepeatSubmitAspect
 * @date 2023/5/23 下午1:13
 * @description 参数式防重复的核心就是IP地址+类+方法+账号的方式，
 * 增加到redis中做为key。第一次加锁成功返回true，第二次返回false，通过这种来做到的防重复。
 */
@Aspect
@Component
@Slf4j
public class RepeatSubmitAspect {

    @Resource
    private RedisUtil redisUtil;

    /**
     * 定义 @Pointcut注解表达式, 通过特定的规则来筛选连接点, 就是Pointcut，选中那几个你想要的方法
     * 在程序中主要体现为书写切入点表达式（通过通配、正则表达式）过滤出特定的一组 JointPoint连接点
     * 方式一：@annotation：当执行的方法上拥有指定的注解时生效（本博客采用这）
     * 方式二：execution：一般用于指定方法的执行
     */
    @Pointcut("@annotation(repeatSubmit)")
    public void pointCutNoRepeatSubmit(RepeatSubmit repeatSubmit) {

    }

    /**
     * 环绕通知, 围绕着方法执行
     *
     * @param joinPoint
     * @param repeatSubmit
     * @return
     * @throws Throwable
     * @Around 可以用来在调用一个具体方法前和调用后来完成一些具体的任务。
     * <p>
     * 方式一：单用 @Around("execution(* net.wnn.controller.*.*(..))")可以
     * 方式二：用@Pointcut和@Around联合注解也可以（本博客采用这个）
     * <p>
     * <p>
     * 两种方式
     * 方式一：加锁 固定时间内不能重复提交
     * <p>
     * 方式二：先请求获取token，这边再删除token,删除成功则是第一次提交
     */
    @Around("pointCutNoRepeatSubmit(repeatSubmit)")
    public Object around(ProceedingJoinPoint joinPoint, RepeatSubmit repeatSubmit) throws Throwable {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        //用于记录成功或者失败
        boolean res = false;
        //防重提交类型
        String type = repeatSubmit.limitType().name();
        if (type.equalsIgnoreCase(RepeatSubmit.Type.PARAM.name())) {
            //方式一，参数形式防重提交
            long lockTime = repeatSubmit.lockTime();
            String ipAddr = IPUtils.getIpAddr(request);
            MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
            Method method = methodSignature.getMethod();
            String className = method.getDeclaringClass().getName();
            String key = "repeat_submit:interface:" + SecureUtil.md5(String.format("%s-%s-%s", ipAddr, className, method));
            //加锁
            // 这种也可以
            res = redisUtil.setIfAbsent(key, "1", lockTime, repeatSubmit.timeUnit());
//            RLock lock = redissonClient.getLock(key);
//            // 尝试加锁，最多等待0秒，上锁以后5秒自动解锁 [lockTime默认为5s, 可以自定义]
//            res = lock.tryLock(0, lockTime, TimeUnit.SECONDS);
        } else {
            //方式二，令牌形式防重提交
            String requestToken = request.getHeader("Authorization");
            if (Preconditions.isBlank(requestToken)) {
                throw new ServiceException("账号或密码输入错误");
            }
            String accountId = JWTUtils.getUserId(requestToken);
            if (StringUtils.isEmpty(accountId)) {
                throw new CommonException(RestCode.UNAUTHORIZED);
            }
            String key = String.format("repeat_submit:token:%s:%s", accountId, requestToken);
            /**
             * 提交表单的token key
             * 方式一：不用lua脚本获取再判断，之前是因为 key组成是 order:submit:accountNo, value是对应的token，所以需要先获取值，再判断
             * 方式二：可以直接key是 order:submit:accountNo:token,然后直接删除成功则完成
             */
            res = redisUtil.del(key);
        }
        if (!res) {
            log.error("请求重复提交");
            throw new ServiceException(String.format("%s%s内禁止重复提交", repeatSubmit.lockTime(), repeatSubmit.timeUnit() == TimeUnit.SECONDS ? "秒" : "毫秒"));
        }
        Object obj = joinPoint.proceed();
        log.info("请求通过");
        return obj;
    }

}
