package com.yx.face.boot.uitls;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdcardUtil;
import com.yx.face.model.enums.SexEnum;
import org.apache.commons.lang3.StringUtils;

import java.util.Calendar;
import java.util.Date;
import java.util.regex.Pattern;

/**
 * @author cyh
 * @Description
 * @date 2021/4/9 13:58
 */
public class SMSOrIdCardUtils {

    //校验手机是否合规 2020年最全的国内手机号格式
    private static final String REGEX_MOBILE = "^1\\d{10}$";

    //校验身份证号码
    private static final String REGEX_ID = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" + "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";


    /**
     * 校验手机号
     *
     * @param phone 手机号
     * @return boolean true:是  false:否
     */
    public static boolean isMobile(String phone) {
        return Pattern.matches(REGEX_MOBILE, phone);
    }

    /**
     * 校验身份证
     *
     * @param IDNumber
     * @return boolean true:是  false:否
     */
    public static boolean isIdCard(String IDNumber) {
        if (IDNumber == null || "".equals(IDNumber)) {
            return false;
        }
        boolean matches = IDNumber.matches(REGEX_ID);

        //判断第18位校验值
        if (matches) {

            if (IDNumber.length() == 18) {
                try {
                    char[] charArray = IDNumber.toCharArray();
                    //前十七位加权因子
                    int[] idCardWi = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
                    //这是除以11后，可能产生的11位余数对应的验证码
                    String[] idCardY = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
                    int sum = 0;
                    for (int i = 0; i < idCardWi.length; i++) {
                        int current = Integer.parseInt(String.valueOf(charArray[i]));
                        int count = current * idCardWi[i];
                        sum += count;
                    }
                    char idCardLast = charArray[17];
                    int idCardMod = sum % 11;
                    if (idCardY[idCardMod].equals(String.valueOf(idCardLast))) {
                        return true;
                    } else {
                        return false;
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    return false;
                }
            }
        }
        return matches;
    }

    /*
    隐藏手机号中间的部分
     */
    public static String hidePhone(String phone) {
        if (StringUtils.isBlank(phone) || phone.length() != 11) {
            return phone;
        }
        return phone.replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
    }


    /*
    隐藏身份证中间的部分
     */
    public static String hideIdNumber(String idNumber) {
        if (!IdcardUtil.isValidCard(idNumber)) {
            return idNumber;
        }
        return IdcardUtil.hide(idNumber, 4, 14);
    }

    /**
     * 隐藏场所码中间的部分
     *
     * @param addressCode
     * @return
     */
    public static String hideAddressCode(String addressCode) {
        if (StringUtils.isBlank(addressCode)) {
            return addressCode;
        }
        return addressCode.substring(0, 4) + "*****" + addressCode.substring(14);
    }

    /*
    隐藏姓名  的名部分
    */
    public static String hideName(String name) {
        return DesensitizedUtil.chineseName(name);
    }


    /*
     * 根据身份证号获取年龄
     * */
    public static int getAge(String idCard) {
        String birthday = idCard.substring(6, 14);
        Calendar now = Calendar.getInstance();
        int nowYear = now.get(Calendar.YEAR);
        int nowMonth = (now.get(Calendar.MONTH) + 1);
        int nowDay = now.get(Calendar.DAY_OF_MONTH);

        int year = Integer.parseInt(birthday.substring(0, 4));
        int month = Integer.parseInt(birthday.substring(4, 6));
        int day = Integer.parseInt(birthday.substring(6));


        int age = nowYear - year;
        if (age <= 0 || age >= 100) {
            return -1;
        }
        if (nowMonth - month > 0) {
            return age;
        }
        if (nowMonth - month == 0) {
            if (nowDay - day >= 0) {
                return age;
            }
        }
        return age - 1;
    }

    /**
     * 通过身份证号获取性别
     *
     * @param identifyNumber
     * @return 若身份证号不规范泽返回null
     */
    public static SexEnum getGender(String identifyNumber) {

        if (Preconditions.isNotBlank(identifyNumber)) {
            String gender;

            if (identifyNumber.length() == 15) {
                gender = identifyNumber.substring(14, 15);
            } else if (identifyNumber.length() == 18) {
                gender = identifyNumber.substring(16, 17);
            } else {
                return null;
            }
            // 奇数为男 偶数为女
            return Integer.parseInt(gender) % 2 == 0 ? SexEnum.FEMALE : SexEnum.MALE;
        }

        return null;
    }

    /**
     * 从身份证中获取出生日期
     *
     * @param idcard
     * @return 当解析失败（身份证为空，身份证不正确等）时，返回empty
     */
    public static String getBirthDate(String idcard) {
        if (Preconditions.isBlank(idcard)) {
            return null;
        }

        if (idcard.length() == 18) {
            try {
                Date d = DateUtils.parse(idcard.substring(6, 14), DateUtils.YYYY_MM_DD);
                return DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, d);
            } catch (Exception e) {
                return null;
            }
        }

        if (idcard.length() == 15) {
            try {
                Date d = DateUtils.parse("19" + idcard.substring(6, 12), DateUtils.YYYY_MM_DD);
                return DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, d);
            } catch (Exception e) {
                return null;
            }
        }
        return null;
    }

    /**
     * 根据年月日计算真实年龄
     *
     * @param birthday 1997-09-09
     * @return
     */
    public static int calAgeByBirthday(String birthday) {
        Date parseBirthDay = DateUtils.parse(birthday, DateUtils.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        if (cal.before(parseBirthDay)) { //出生日期晚于当前时间，无法计算
            throw new IllegalArgumentException("出生日期晚于当前时间，无法计算");
        }
        int yearNow = cal.get(Calendar.YEAR);  //当前年份
        int monthNow = cal.get(Calendar.MONTH);  //当前月份
        int dayOfMonthNow = cal.get(Calendar.DAY_OF_MONTH); //当前日期
        cal.setTime(parseBirthDay);
        int yearBirth = cal.get(Calendar.YEAR);
        int monthBirth = cal.get(Calendar.MONTH);
        int dayOfMonthBirth = cal.get(Calendar.DAY_OF_MONTH);
        int age = yearNow - yearBirth;   //计算整岁数
        if (monthNow <= monthBirth) {
            if (monthNow == monthBirth) {
                if (dayOfMonthNow < dayOfMonthBirth) age--;//当前日期在生日之前，年龄减一
            } else {
                age--;//当前月份在生日之前，年龄减一
            }
        }
        return age;
    }

    public static void main(String[] args) {
        System.out.println(calAgeByBirthday("1993-03-01"));
    }

}
