package com.yx.face.boot.uitls;

import com.yx.face.boot.core.Constants;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;

/**
 * @description: 图片压制至小于指定大小
 * @ClassName ThumbnailsUtil
 * @Author WXG
 * @Date 2021/10/20 15:28
 */
@Slf4j
public class ThumbnailsUtil {
    private static final Integer ZERO = 0;
    private static final Integer ONE_ZERO_TWO_FOUR = 1024;
    private static final Integer NINE_ZERO_ZERO = 900;
    private static final Integer THREE_TWO_SEVEN_FIVE = 3275;
    private static final Integer TWO_ZERO_FOUR_SEVEN = 2047;
    private static final Double ZERO_EIGHT_FIVE = 0.85;
    private static final Double ZERO_SIX = 0.6;
    private static final Double ZERO_FOUR_FOUR = 0.44;
    private static final Double ZERO_FOUR = 0.4;


    public static byte[] compressPicForScale(byte[] imageBytes, long desFileSize) {
        if (imageBytes == null || imageBytes.length <= ZERO || imageBytes.length < desFileSize * ONE_ZERO_TWO_FOUR) {
            return imageBytes;
        }
        long srcSize = imageBytes.length;
        double accuracy = getAccuracy(srcSize / ONE_ZERO_TWO_FOUR);
        try {
            while (imageBytes.length > desFileSize * ONE_ZERO_TWO_FOUR) {
                ByteArrayInputStream inputStream = new ByteArrayInputStream(imageBytes);
                ByteArrayOutputStream outputStream = new ByteArrayOutputStream(imageBytes.length);
                Thumbnails.of(inputStream)
                        .scale(accuracy)
                        .outputQuality(accuracy)
                        .toOutputStream(outputStream);
                imageBytes = outputStream.toByteArray();
            }
            log.info("图片原大小={}kb | 压缩后大小={}kb",
                    srcSize / ONE_ZERO_TWO_FOUR, imageBytes.length / ONE_ZERO_TWO_FOUR);
        } catch (Exception e) {
            log.error("【图片压缩】msg=图片压缩失败!", e);
        }
        return imageBytes;
    }

    public static double getAccuracy(long size) {
        double accuracy;
        if (size < NINE_ZERO_ZERO) {
            accuracy = ZERO_EIGHT_FIVE;
        } else if (size < TWO_ZERO_FOUR_SEVEN) {
            accuracy = ZERO_SIX;
        } else if (size < THREE_TWO_SEVEN_FIVE) {
            accuracy = ZERO_FOUR_FOUR;
        } else {
            accuracy = ZERO_FOUR;
        }
        return accuracy;
    }


    /**
     * 对图片进行原比例无损压缩,压缩后覆盖原图片
     *
     * @param path
     * @param maxkb
     */
    public static byte[] compressCycle(String path, Integer maxkb) {
        try {
            return compressPhoto(new File(path), maxkb, null, null, 0.9f);
        } catch (Exception e) {
            log.error("压缩图片失败!", e);
        }
        return null;
    }

    public static byte[] compressCycle(byte[] bytes, Integer maxkb) {
        try {
            return compressPhoto(bytes, maxkb, null, null, 0.9f);
        } catch (Exception e) {
            log.error("压缩图片失败!", e);
        }
        return null;
    }

    public static byte[] compressPhoto(byte[] bytes, Integer maxkb, Integer photoWidth, Integer photoHeight, Float quality) throws Exception {

        // 2.是否超過100kb?沒超過就不處理圖片
        long fileSize = bytes.length;
        if (fileSize <= maxkb * 1024) {
            log.info("图片不超过{}kb,无需压缩。", maxkb);
            return bytes;
        }

        // 3.压缩到300-500宽高，100kb
        BufferedImage bim = ImageIO.read(new ByteArrayInputStream(bytes));
        int imgWidth = bim.getWidth();
        int imgHeight = bim.getHeight();

        // 3.1.先按宽高判断是否需要缩放  outputQuality=1确保画质清晰
        if (photoWidth != null && photoHeight != null && imgWidth >= photoWidth && imgHeight >= photoHeight) {
            log.info("先按宽{}高{}压缩。", photoWidth, photoHeight);
            ByteArrayOutputStream out = new ByteArrayOutputStream();//Closing a <tt>ByteArrayOutputStream</tt> has no effect.   因此無需close
            Thumbnails.of(new ByteArrayInputStream(bytes)).size(photoWidth, photoHeight).outputQuality(1).outputFormat("jpg").toOutputStream(out);
            bytes = out.toByteArray();
        }
        // 3.2.判断按宽高缩放后是否超过100kb,超过就按质量压缩（图片会变得更模糊！）
//        quality = (float) getAccuracy(bytes.length / 1024);
        bytes = compressPhotoByQuality(bytes, quality, maxkb);
        return bytes;
    }

    public static void main(String[] args) {
        byte[] bytes = compressCycle("/Users/leon/hanghui_workspace/face-server/file/upload/imagesnew/20230915/202007011329308612.jpg", 50);
        String filename = "202007011329308612.jpg";
        String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW);
        try {
            FileUtils.saveByCopy(filename, filePath, new ByteArrayInputStream(bytes));
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    /**
     * 【压缩图片】到要求的图片标准:  宽[300,500],高[300,500];100kb;jpg/jpeg
     * 先按宽高压缩，压缩后如果还不满足100kb,則按质量递归压缩
     *
     * @param imgFile     源原文件
     * @param maxkb       目标kb
     * @param photoWidth  目标寬度 172px
     * @param photoHeight 模板高度 240px
     * @param quality     质量
     * @return 处理后的文件byte[]
     * @throws Exception
     * @Title:compressPhoto
     * @Description: TODO
     */
    public static byte[] compressPhoto(File imgFile, Integer maxkb, Integer photoWidth, Integer photoHeight, Float quality) throws Exception {
        // 1.压缩图片是否存在
        if (imgFile == null || !imgFile.exists()) {
            throw new Exception("图片文件不存在。");
        }
        byte[] bytes = readFileToByteArray(imgFile);
        if (bytes == null || bytes.length == 0) {
            throw new Exception("图片文件为空。");
        }
        return compressPhoto(bytes, maxkb, photoWidth, photoHeight, quality);
    }

    /**
     * 遞歸按quality質量處理，壓縮到maxkb後返回新的bytes值
     *
     * @param bytes   源文件字節
     * @param quality 壓縮質量 （如果爲1則不處理）
     * @param maxkb   要求壓縮到的最大kb
     * @return
     * @throws IOException
     * @Title:compressPhotoByQuality
     * @Description: TODO
     * @date 2020年6月30日 下午2:24:36
     * @author yqwang
     */
    public static byte[] compressPhotoByQuality(byte[] bytes, Float quality, long maxkb) throws IOException {
        if (bytes == null) {
            return bytes;
        }
        log.info("开始按质量压缩图片({}kb)。", bytes.length / 1024);
        // 如果配置的1，則不再处理
        if (quality == 1) {
            log.info("quality=1,不执行压缩。");
            return bytes;
        }
        // 滿足目標kb值，則返回
        long fileSize = bytes.length;
        if (fileSize <= maxkb * 1024) {
            log.info("图片文件{}kb<={}kb,不再压缩质量。", fileSize / 1024, maxkb);
            return bytes;
        }
        // Closing a <tt>ByteArrayOutputStream</tt> has no effect.   因此無需close
        ByteArrayOutputStream out = null;
        out = new ByteArrayOutputStream();
        BufferedImage bim = ImageIO.read(new ByteArrayInputStream(bytes));
        int imgWidth = bim.getWidth();
        int imgHeight = bim.getHeight();
        // 如果不處理size,只用quality,可能導致一致壓縮不到目標值，一致遞歸在當前方法中！！
        int desWidth = new BigDecimal(imgWidth).multiply(new BigDecimal(quality)).intValue();
        int desHeight = new BigDecimal(imgHeight).multiply(new BigDecimal(quality)).intValue();
        log.info("图片将按照width={}*height={}进行压缩，画质quality={}。", desWidth, desHeight, quality);
        Thumbnails.of(new ByteArrayInputStream(bytes)).size(desWidth, desHeight).outputQuality(quality).outputFormat("jpg").toOutputStream(out);
        //遞歸
        return compressPhotoByQuality(out.toByteArray(), quality, maxkb);
    }

    /**
     * File to bytes[]
     */
    public static byte[] readFileToByteArray(File f) throws Exception {
        byte[] fileb = null;
        InputStream is = null;
        ByteArrayOutputStream out = new ByteArrayOutputStream();

        try {
            is = new FileInputStream(f);
            byte[] b = new byte[1024];

            int n;
            while ((n = is.read(b)) != -1) {
                out.write(b, 0, n);
            }

            fileb = out.toByteArray();
            return fileb;
        } catch (Exception var16) {
            throw new Exception("readFileToByteArray", var16);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (Exception var15) {
                    ;
                }
            }

            if (out != null) {
                try {
                    out.close();
                } catch (Exception var14) {
                }
            }

        }
    }


}
