package com.yx.face.controller.admin;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ExcelListenerUtils;
import com.yx.face.model.dto.AdminDTO;
import com.yx.face.model.dto.EnterpriseAccountDTO;
import com.yx.face.model.entity.AdminPlaceInfo;
import com.yx.face.model.entity.RiskArea;
import com.yx.face.model.enums.PlaceAreaEnum;
import com.yx.face.model.excel.AdminInfoExcel;
import com.yx.face.model.search.AdminListNewSearch;
import com.yx.face.model.search.AdminSearch;
import com.yx.face.model.update.AdminUpdate;
import com.yx.face.model.vo.AdminCodeVO;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.Pattern;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 管理人员信息表(Admin)表控制层 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-05-21 13:37:01 <br>
 */

@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 管理人员权限相关功能API")
@RestController
@RequestMapping("admin/auth")
@Validated
public class AdminController extends BaseController {

    @Value("#{${dict.punch-report-optional-fields}}")
    private Map<String, String> dictPunchReportOptionalFields;

    /**
     * 服务对象
     */
    private final AdminService adminService;

    @ApiOperation(value = "登录")
    @Authority(required = false)
    @GetMapping("doLogin")
    public RestResult<AdminVO> doLogin(@RequestParam String username,
                                       @Pattern(
                                               regexp="^(((?=.*\\d)(?=.*[a-z])(?=.*[A-Z]))|" +
                                                       "((?=.*\\d)(?=.*[a-z])(?=.*[~!@#$%^&*]))|" +
                                                       "((?=.*\\d)(?=.*[A-Z])(?=.*[~!@#$%^&*]))|" +
                                                       "((?=.*[a-z])(?=.*[A-Z])(?=.*[~!@#$%^&*]))|" +
                                                       "((?=.*\\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[~!@#$%^&*]))).{10,32}$",
                                               message = "密码至少包含大小写字母、数字、特殊字符中的3种，且密码长度在10-32位之间"
                                       ) @RequestParam String password) {
        return RestResponse.ok(adminService.login(username, password));
    }

    @ApiOperation(value = "退出")
    @Authority(required = false)
    @GetMapping("doLogout")
    public RestResult<Boolean> doLogout() {
        return RestResponse.ok(adminService.logout());
    }


    /**
     * @description: 获取管理员信息 <br>
     * @date: 2021/5/4 17:08 <br>
     * @author: PWB <br>
     */
    @ApiOperation(value = "获取管理员信息")
    @GetMapping("doGetInfo")
    public RestResult<AdminVO> doGetInfo() {
        return RestResponse.ok(adminService.getInfo(getUserId()));
    }

    @ApiOperation(value = "行程码-行政区域列表列表")
    @PostMapping("riskAreaList")
    public RestResult<List<RiskArea>> riskAreaList(@RequestBody RiskArea riskArea) {
        return RestResponse.ok(adminService.riskAreaList(riskArea));
    }

    /**
     * @param dto <br>
     * @description: 添加/编辑管理员 <br>
     * @date: 2021/5/4 17:09 <br>
     * @author: PWB <br>
     */
    @ApiOperation(value = "添加/编辑管理员")
    @Authority(value = {1, 2})
    @PostMapping("doEditAdmin")
    public RestResult<Boolean> doEditAdmin(@RequestBody AdminDTO dto) {
        return RestResponse.ok(adminService.editAdmin(dto));
    }

    /**
     * @param dto <br>
     * @description: 获取管理员列表 <br>
     * @date: 2021/5/4 17:09 <br>
     * @author: PWB <br>
     */
    @ApiOperation(value = "获取管理员列表")
    @PostMapping("doGetAdmins")
    @Authority(value = {1, 2, 3, 4, 5})
    public RestResult<PageInfo<AdminVO>> doGetAdmins(@RequestBody RestDTO<AdminSearch> dto) {
        return RestResponse.ok(adminService.getAdmins(dto));
    }

    @ApiOperation(value = "通过账号id查询详情")
    @GetMapping("/queryAdminById/{id}")
    public RestResult<EnterpriseAccountDTO> queryAdminById(@PathVariable("id") Integer id) {
        return RestResponse.ok(adminService.queryAdminById(id));
    }

    @ApiOperation(value = "查询企业账号信息")
    @GetMapping("/queryEnterpriseAccount")
    public RestResult<List<EnterpriseAccountDTO>> queryEnterpriseAccount() {
        return RestResponse.ok(adminService.queryEnterpriseAccount(getUserId()));
    }

    @ApiOperation(value = "获取管理员列表(id 和 username)")
    @PostMapping("getAdminlist")
    public RestResult<List<AdminVO>> getAdminlist() {
        return RestResponse.ok(adminService.getAdminlist());
    }

    @ApiOperation(value = "(新)获取管理员列表")
    @PostMapping("getAdminListNew")
    public RestResult<List<AdminVO>> getAdminListNew(@RequestBody AdminListNewSearch adminListNewSearch) {
        return RestResponse.ok(adminService.getAdminListNew(adminListNewSearch));
    }

    /**
     * @param id <br>
     * @description: 删除管理员 <br>
     * @date: 2021/5/4 17:09 <br>
     * @author: PWB <br>
     */
    @ApiOperation(value = "删除管理员")
    @Authority(value = {1, 2, 5})
    @DeleteMapping("doDeleteAdmin")
    public RestResult<Boolean> doDeleteAdmin(@RequestParam Long id) {
        return RestResponse.ok(adminService.deleteAdmin(id));
    }

    @ApiOperation("管理员的批量导入")
    @PostMapping("uploadAdminInfoExcel")
    public RestResult<List<String>> uploadAdminInfoExcel(@RequestParam("file") MultipartFile excelFile) {
        ExcelListenerUtils listener = new ExcelListenerUtils();  //ExcelListener
        try {
            EasyExcel.read(excelFile.getInputStream(), AdminInfoExcel.class, listener).sheet(0).doReadSync();  //ExcelModel 上面创建的实体类
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<Object> datas = listener.getDatas(); //把拿到的数据处理到集合里去(根据所需自行修改)
        return adminService.uploadAdminInfoExcel(datas);
    }

    @ApiOperation(value = "管理员修改自己的信息")
    @PostMapping("updateAdmin")
    public RestResult<Boolean> updateAdmin(@RequestBody AdminUpdate adminUpdate) {
        return RestResponse.ok(adminService.updateAdmin(adminUpdate));
    }

    @ApiOperation(value = "二维码类型记录标识")
    @GetMapping("/getCodeType")
    public RestResult<List<AdminCodeVO>> getCodeType() {
        return RestResponse.ok(adminService.getCodeType());
    }

    @ApiOperation(value = "添加/编辑企业")
    @PostMapping("/saveEnterpriseAccount")
    public RestResult<Boolean> saveEnterpriseAccount(@RequestBody EnterpriseAccountDTO dto) {
        return RestResponse.ok(adminService.saveEnterpriseAccount(dto));
    }

    @ApiOperation(value = "企业审核")
    @PostMapping("/audit/{id}/{reviewStatus}")
    public RestResult<Boolean> audit(@PathVariable("id") Integer id, @PathVariable("reviewStatus") Integer reviewStatus) {
        return RestResponse.ok(adminService.audit(id, reviewStatus));
    }

    @ApiOperation(value = "查询账号场所码信息")
    @GetMapping("/queryPlaceInfos/{adminId}")
    public RestResult<List<AdminPlaceInfo>> queryPlaceInfos(@PathVariable Integer adminId) {
        return RestResponse.ok(adminService.queryPlaceInfos(adminId));
    }

    /**
     * key: 字典值 value: 字典描述
     *
     * @return
     */
    @Authority(required = false)
    @ApiOperation(value = "获取打卡推送可选字段字典项")
    @GetMapping("/punch/field-options")
    public RestResult<List<JSONObject>> getPunchFieldOptions() {
        return RestResponse.ok(dictPunchReportOptionalFields.entrySet().stream().map(item ->
                JSONUtil.createObj().set("value", item.getKey()).set("label", item.getValue())).collect(Collectors.toList()));
    }

    @ApiOperation(value = "获取场所码区域配置 PlaceAreaEnum 0:浙江 1:嘉兴 2:绍兴")
    @GetMapping("/getPlaceAreas")
    public RestResult<List<com.alibaba.fastjson.JSONObject>> getPlaceAreas() {
        return RestResponse.ok(PlaceAreaEnum.getList());
    }

}

