package com.yx.face.server;

import cn.hutool.core.util.IdUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.boot.restful.RestCode;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.server.dto.DeviceBindManagerDTO;
import com.yx.face.server.dto.DevicePunchCardDTO;
import com.yx.face.server.dto.DeviceRegisterDTO;
import com.yx.face.server.dto.TokenDataDTO;
import com.yx.face.server.vo.HealthResultVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import static com.yx.face.model.enums.PlaceAreaEnum.getByPlaceAreaCode;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Component
@Slf4j
public class PlaceHealthCodeServer {
    private static final String ZHE_JIANG_TOKEN_KEY = "health:code:token";
    private static final String JIA_XING_TOKEN_KEY = "jia:health:code:token";
    private static final String SHAO_XING_TOKEN_KEY = "shao:health:code:token";

    @Value("${health.code.zheClientId}")
    private String zheClientId;

    @Value("${health.code.zheClientSecret}")
    private String zheClientSecret;

    @Value("${health.code.jiaClientId}")
    private String jiaClientId;

    @Value("${health.code.jiaClientSecret}")
    private String jiaClientSecret;

    @Value("${health.code.shaoClientId}")
    private String shaoClientId;

    @Value("${health.code.shaoClientSecret}")
    private String shaoClientSecret;

    @Value("${health.code.zheDomain}")
    private String zheDomain;

    @Value("${health.code.jiaDomain}")
    private String jiaDomain;

    @Value("${health.code.shaoDomain}")
    private String shaoDomain;

    @Value("${health.code.getTokenUri}")
    private String getTokenUri;

    @Value("${health.code.deviceRegisterUri}")
    private String deviceRegisterUri;

    @Value("${health.code.deviceBindManagerUri}")
    private String deviceBindManagerUri;

    @Value("${health.code.devicePunchCardUri}")
    private String devicePunchCardUri;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession, Integer placeAreaCode) {
        String TOKEN_KEY = convertTokenKey(placeAreaCode);
        if (cleanSession != null && cleanSession) {
            redisUtil.del(TOKEN_KEY);
        }

        if (redisUtil.get(TOKEN_KEY) != null) {
            HealthResultVO healthResultVO = JSON.parseObject(redisUtil.get(TOKEN_KEY).toString(), HealthResultVO.class);
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
            return healthResultVO;
        }

        long timeMillis = System.currentTimeMillis();
        String secret = SecureUtil.md5(new StringBuilder().append(timeMillis).append(convertClientSecret(placeAreaCode)).toString().toLowerCase());
        String tokenUrl = new StringBuilder().append(getDomain(placeAreaCode)).append(getTokenUri).append("?clientId=").append(convertClientId(placeAreaCode)).append("&time=").append(timeMillis).append("&secret=").append(secret).toString();
        HealthResultVO healthResultVO = restTemplate.getForObject(tokenUrl, HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        if (healthResultVO.getSuccess() && healthResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            redisUtil.set(TOKEN_KEY, JSON.toJSONString(healthResultVO), 90 * 60);
        }

        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Integer placeAreaCode) {
        return this.getToken(false, placeAreaCode);
    }

    /**
     * 设备注册
     *
     * @return
     */
    public HealthResultVO deviceRegister(DeviceRegisterDTO dto) {
        //认证token
        String tokenKey = convertTokenKey(dto.getPlaceAreaCode());
        String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
        dto.setToken(token);
        dto.setClientId(convertClientId(dto.getPlaceAreaCode()));
        dto.setBizNo(IdUtil.fastUUID());
        String deviceRegisterUrl = new StringBuilder().append(getDomain(dto.getPlaceAreaCode())).append(deviceRegisterUri).toString();
        HealthResultVO resultVO = restTemplate.postForObject(deviceRegisterUrl, dto, HealthResultVO.class);
        log.info("设备注册响应内容：{}", JSON.toJSONString(resultVO));
        if (resultVO.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
            //token过期本地缓存删除
            redisUtil.del(tokenKey);
        }

        return resultVO;
    }

    /**
     * 绑定管理人员
     *
     * @return
     */
    public HealthResultVO deviceBindManager(DeviceBindManagerDTO dto) {
        //认证token
        String tokenKey = convertTokenKey(dto.getPlaceAreaCode());
        String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
        dto.setToken(token);
        dto.setBizNo(IdUtil.fastUUID());
        String deviceBindManagerUrl = new StringBuilder().append(getDomain(dto.getPlaceAreaCode())).append(deviceBindManagerUri).toString();
        HealthResultVO resultVO = restTemplate.postForObject(deviceBindManagerUrl, dto, HealthResultVO.class);
        log.info("绑定管理人员响应内容：{}", JSON.toJSONString(resultVO));
        if (resultVO.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
            //token过期本地缓存删除
            redisUtil.del(tokenKey);
        }
        return resultVO;
    }

    /**
     * 绑定地址码
     *
     * @returne
     */
    public HealthResultVO devicePunchCard(DevicePunchCardDTO dto) {
        //认证token
        String tokenKey = convertTokenKey(dto.getPlaceAreaCode());
        String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
        dto.setToken(token);
        dto.setBizNo(IdUtil.fastUUID());
        String devicePunchCardUrl = new StringBuilder().append(getDomain(dto.getPlaceAreaCode())).append(devicePunchCardUri).toString();
        HealthResultVO resultVO = restTemplate.postForObject(devicePunchCardUrl, dto, HealthResultVO.class);
        log.info("绑定地址码响应内容：{}", JSON.toJSONString(resultVO));
        if (resultVO.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
            //token过期本地缓存删除
            redisUtil.del(tokenKey);
        }
        return resultVO;
    }

    private String getDomain(Integer placeAreaCode) {
        String domain = zheDomain;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    domain = jiaDomain;
                    break;
                case SHAO_XING:
                    //绍兴
                    domain = shaoDomain;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return domain;
    }

    private String convertTokenKey(Integer placeAreaCode) {
        String tokenKey = ZHE_JIANG_TOKEN_KEY;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    tokenKey = JIA_XING_TOKEN_KEY;
                    break;
                case SHAO_XING:
                    //绍兴
                    tokenKey = SHAO_XING_TOKEN_KEY;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return tokenKey;
    }

    private String convertClientId(Integer placeAreaCode) {
        String clientId = zheClientId;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    clientId = jiaClientId;
                    break;
                case SHAO_XING:
                    //绍兴
                    clientId = shaoClientId;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return clientId;
    }

    private String convertClientSecret(Integer placeAreaCode) {
        String clientSecret = zheClientSecret;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    clientSecret = jiaClientSecret;
                    break;
                case SHAO_XING:
                    //绍兴
                    clientSecret = shaoClientSecret;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return clientSecret;
    }


}
